From fbe54043cf598b19a753dc2b21a7ed558d23fd15 Mon Sep 17 00:00:00 2001
From: thomascube <thomas@roundcube.net>
Date: Thu, 18 Aug 2011 13:40:07 -0400
Subject: [PATCH] Fix folders drop-down list: descend into root folder (e.g. INBOX) even if part of the exception list

---
 program/include/rcube_user.php |  115 +++++++++++++++++++++++++++++++++++++++++----------------
 1 files changed, 82 insertions(+), 33 deletions(-)

diff --git a/program/include/rcube_user.php b/program/include/rcube_user.php
index abd5313..d2bbaa1 100644
--- a/program/include/rcube_user.php
+++ b/program/include/rcube_user.php
@@ -5,7 +5,7 @@
  | program/include/rcube_user.inc                                        |
  |                                                                       |
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2005-2010, Roundcube Dev. - Switzerland                 |
+ | Copyright (C) 2005-2010, The Roundcube Dev Team                       |
  | Licensed under the GNU GPL                                            |
  |                                                                       |
  | PURPOSE:                                                              |
@@ -29,21 +29,35 @@
  */
 class rcube_user
 {
-    public $ID = null;
-    public $data = null;
-    public $language = null;
+    public $ID;
+    public $data;
+    public $language;
 
-    private $db = null;
+    /**
+     * Holds database connection.
+     *
+     * @var rcube_mdb2
+     */
+    private $db;
+
+    /**
+     * rcmail object.
+     *
+     * @var rcmail
+     */
+    private $rc;
 
 
     /**
      * Object constructor
      *
-     * @param object DB Database connection
+     * @param int   $id      User id
+     * @param array $sql_arr SQL result set
      */
     function __construct($id = null, $sql_arr = null)
     {
-        $this->db = rcmail::get_instance()->get_dbh();
+        $this->rc = rcmail::get_instance();
+        $this->db = $this->rc->get_dbh();
 
         if ($id && !$sql_arr) {
             $sql_result = $this->db->query(
@@ -62,7 +76,7 @@
     /**
      * Build a user name string (as e-mail address)
      *
-     * @param string Username part (empty or 'local' or 'domain')
+     * @param  string $part Username part (empty or 'local' or 'domain')
      * @return string Full user name or its part
      */
     function get_username($part = null)
@@ -76,8 +90,7 @@
             }
             // if no domain was provided...
             if (empty($domain)) {
-                $rcmail = rcmail::get_instance();
-                $domain = $rcmail->config->mail_domain($this->data['mail_host']);
+                $domain = $this->rc->config->mail_domain($this->data['mail_host']);
             }
 
             if ($part == 'domain') {
@@ -104,8 +117,25 @@
         if (!empty($this->language))
             $prefs = array('language' => $this->language);
 
-        if ($this->ID && $this->data['preferences'])
-            $prefs += (array)unserialize($this->data['preferences']);
+        if ($this->ID) {
+            // Preferences from session (write-master is unavailable)
+            if (!empty($_SESSION['preferences'])) {
+                // Check last write attempt time, try to write again (every 5 minutes)
+                if ($_SESSION['preferences_time'] < time() - 5 * 60) {
+		    $saved_prefs = unserialize($_SESSION['preferences']);
+                    $this->rc->session->remove('preferences');
+	            $this->rc->session->remove('preferences_time');
+                    $this->save_prefs($saved_prefs);
+                }
+                else {
+                    $this->data['preferences'] = $_SESSION['preferences'];
+                }
+            }
+
+            if ($this->data['preferences']) {
+                $prefs += (array)unserialize($this->data['preferences']);
+            }
+        }
 
         return $prefs;
     }
@@ -114,7 +144,7 @@
     /**
      * Write the given user prefs to the user's record
      *
      * @param array User prefs to save
      * @return boolean True on success, False on failure
      */
     function save_prefs($a_user_prefs)
@@ -122,7 +152,7 @@
         if (!$this->ID)
             return false;
 
-        $config = rcmail::get_instance()->config;
+        $config    = $this->rc->config;
         $old_prefs = (array)$this->get_prefs();
 
         // merge (partial) prefs array with existing settings
@@ -148,10 +178,25 @@
 
         $this->language = $_SESSION['language'];
 
-        if ($this->db->affected_rows()) {
+        // Update success
+        if ($this->db->affected_rows() !== false) {
             $config->set_user_prefs($a_user_prefs);
             $this->data['preferences'] = $save_prefs;
+
+            if (isset($_SESSION['preferences'])) {
+                $this->rc->session->remove('preferences');
+                $this->rc->session->remove('preferences_time');
+            }
             return true;
+        }
+        // Update error, but we are using replication (we have read-only DB connection)
+        // and we are storing session not in the SQL database
+        // we can store preferences in session and try to write later (see get_prefs())
+        else if ($this->db->is_replicated() && $config->get('session_storage', 'db') != 'db') {
+            $_SESSION['preferences'] = $save_prefs;
+            $_SESSION['preferences_time'] = time();
+            $config->set_user_prefs($a_user_prefs);
+            $this->data['preferences'] = $save_prefs;
         }
 
         return false;
@@ -161,7 +206,7 @@
     /**
      * Get default identity of this user
      *
-     * @param int  Identity ID. If empty, the default identity is returned
+     * @param  int   $id Identity ID. If empty, the default identity is returned
      * @return array Hash array with all cols of the identity record
      */
     function get_identity($id = null)
@@ -174,6 +219,7 @@
     /**
      * Return a list of all identities linked with this user
      *
+     * @param string $sql_add Optional WHERE clauses
      * @return array List of identities
      */
     function list_identities($sql_add = '')
@@ -198,8 +244,8 @@
     /**
      * Update a specific identity record
      *
-     * @param int    Identity ID
-     * @param array  Hash array with col->value pairs to save
+     * @param int    $iid  Identity ID
+     * @param array  $data Hash array with col->value pairs to save
      * @return boolean True if saved successfully, false if nothing changed
      */
     function update_identity($iid, $data)
@@ -232,7 +278,7 @@
     /**
      * Create a new identity record linked with this user
      *
-     * @param array  Hash array with col->value pairs to save
+     * @param array $data Hash array with col->value pairs to save
      * @return int  The inserted identity ID or false on error
      */
     function insert_identity($data)
@@ -264,7 +310,7 @@
     /**
      * Mark the given identity as deleted
      *
-     * @param int  Identity ID
+     * @param  int     $iid Identity ID
      * @return boolean True if deleted successfully, false if nothing changed
      */
     function delete_identity($iid)
@@ -281,7 +327,7 @@
 
         // we'll not delete last identity
         if ($sql_arr['ident_count'] <= 1)
-            return false;
+            return -1;
 
         $this->db->query(
             "UPDATE ".get_table_name('identities').
@@ -298,7 +344,7 @@
     /**
      * Make this identity the default one for this user
      *
-     * @param int The identity ID
+     * @param int $iid The identity ID
      */
     function set_default($iid)
     {
@@ -343,16 +389,19 @@
     /**
      * Find a user record matching the given name and host
      *
-     * @param string IMAP user name
-     * @param string IMAP host name
-     * @return object rcube_user New user instance
+     * @param string $user IMAP user name
+     * @param string $host IMAP host name
+     * @return rcube_user New user instance
      */
     static function query($user, $host)
     {
         $dbh = rcmail::get_instance()->get_dbh();
 
+        // use BINARY (case-sensitive) comparison on MySQL, other engines are case-sensitive
+        $mod = preg_match('/^mysql/', $dbh->db_provider) ? 'BINARY' : '';
+
         // query for matching user name
-        $query = "SELECT * FROM ".get_table_name('users')." WHERE mail_host = ? AND %s = ?";
+        $query = "SELECT * FROM ".get_table_name('users')." WHERE mail_host = ? AND %s = $mod ?";
         $sql_result = $dbh->query(sprintf($query, 'username'), $host, $user);
 
         // query for matching alias
@@ -372,9 +421,9 @@
     /**
      * Create a new user record and return a rcube_user instance
      *
-     * @param string IMAP user name
-     * @param string IMAP host
-     * @return object rcube_user New user instance
+     * @param string $user IMAP user name
+     * @param string $host IMAP host
+     * @return rcube_user New user instance
      */
     static function create($user, $host)
     {
@@ -471,7 +520,7 @@
     /**
      * Resolve username using a virtuser plugins
      *
-     * @param string E-mail address to resolve
+     * @param string $email E-mail address to resolve
      * @return string Resolved IMAP username
      */
     static function email2user($email)
@@ -487,9 +536,9 @@
     /**
      * Resolve e-mail address from virtuser plugins
      *
-     * @param string User name
-     * @param boolean If true returns first found entry
-     * @param boolean If true returns email as array (email and name for identity)
+     * @param string $user User name
+     * @param boolean $first If true returns first found entry
+     * @param boolean $extended If true returns email as array (email and name for identity)
      * @return mixed Resolved e-mail address string or array of strings
      */
     static function user2email($user, $first=true, $extended=false)
@@ -114,7 +144,7 @@
      * Write the given user prefs to the user's record
      *
-     * @param array User prefs to save
+     * @param array $a_user_prefs User prefs to save
      * @return boolean True on success, False on failure
      */

--
Gitblit v1.9.1